
#
#
#                      PyT3D : Python preprocessor for T3D                    
#                                                                             
#                            version 0.1 (2015-01-07)                         
#  
#                       Copyright (C)  2015  Jan Stransky                     
#  
#           Czech Technical University, Faculty of Civil Engineering,         
#       Department of Structural Mechanics, 166 29 Prague, Czech Republic     
#  
#  PyT3D is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Lesser General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#  
#  PyT3D is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for
#  more details.
#  
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program. If not, see <http://www.gnu.org/licenses/>.
#

"""`Python <http://www.python.org>`_ preprocessor for `T3D <http://mech.fsv.cvut.cz/~dr/t3d.html>`_, a three-dimensional sequential mesh generator

T3D is a powerfull mesh generator capable to discretize complex 3D domains into triangular and tetrahedral meshes of high quality. However, it uses text input files, which might be tedious and difficult to create manually, especially for complex geometries. `PyT3D <http://mech.fsv.cvut.cz/~stransky/en/software/pyt3d/doc/>`_ is a Python module to simplify the process:

For better orientation, see `T3D documentation <http://mech.fsv.cvut.cz/~dr/software/T3d/guide/guide.html>`_ first
"""

try:
	from minieigen import Vector3
except ImportError:
	print "package minieigen <https://pypi.python.org/pypi/minieigen> not found, consider installing it"
	from vector3 import Vector3

DEF = 'def'
UNI = 'uni'
VER = 'ver'
CUR = 'cur'
SUR = 'sur'
MUL = 'mul'

PLUS  = '+'
MINUS = '-'

NVERTICES = 0
NCURVES   = 0
NSURFACES = 0
NPATCHES  = 0
NSHELLS   = 0
NREGIONS  = 0

class PyT3DError(Exception):
	"""Module exception"""
	def __init__(self, msg):
		self.msg = msg
	def __str__(self):
		return repr(self.msg)

def size2str(size,sizeMul):
	"""Auxiliary function converting correct T3D input file size stringable value"""
	if size is None:
		return ''
	ret = 'size '
	if isinstance(size,(int,float)):
		ret += str(size)
	elif size in (DEF,UNI,VER,CUR,SUR):
		ret += size
	elif size == MUL:
		ret += ' * %.8e'%sizeMul
	else:
		raise PyT3DError, "Wrong size parameter"
	return ret

class T3DObject(object):
	"""Base class for all other objects, implementing common interface and attributes

	:param int id: id
	:param size: size
	:param float sizeMul: size multiplier
	""" # TODO
	def __init__(self,
			id = -1,
			size = None,
			sizeMul = None,
			factor = None,
			property = None,
			virtual = None,
			hidden = None,
			output = None,
			coincide = None,
			slave = None,
			master = None,
			noslave = None,
			nomaster = None,
			**kw):
		self.id = id
		"""id attribute"""
		self.size = size
		"""size attribute"""
		self.sizeMul = sizeMul
		"""size multiplier"""
		self.factor = factor
		"""factor attribute"""
		self.property = property
		"""property attribute"""
		self.virtual = virtual
		"""virtual attribute"""
		self.hidden = hidden
		"""hidden attribute"""
		self.output = output
		"""output attribute"""
		self.coincide = coincide
		"""coincide attribute"""
		self.slave = slave
		"""slave attribute"""
		self.master = master
		"""master attribute"""
		self.noslave = noslave
		"""noslave attribute"""
		self.nomaster = nomaster
		"""nomaster attribute"""
		self.isWritten = False
		"""whether receiver is already written or not"""
	def __str__(self):
		"""Returns string representation of receiver"""
		ret = ' ' + size2str(self.size,self.sizeMul)
		if self.factor is not None:
			ret += ' factor %.8e'%self.factor
		if self.property is not None:
			ret += ' property %d'%self.property
		if self.virtual:
			ret += ' virtual'
		if self.hidden:
			ret += ' hidden'
		if self.output is not None:
			ret += ' output %s'%('yes' if self.output else 'no')
		if self.coincide is not None:
			ret += ' coincide vertex %d'%self.coincide.id
		if self.slave:
			ret += ' slave'
		if self.master:
			ret += ' master'
		if self.noslave:
			ret += ' noslave'
		if self.nomaster:
			ret += ' nomaster'
		return ret
	def __hash__(self):
		return id(self)
	def toFile(self,f,deps=[]):
		"""Writes receiver to given file / file name
		
		:param file|str f: opened file for writing of file name
		:param  deps: dependencies to be write prior to receiver
		:type deps: [:class:`T3DObject`]
		"""
		if isinstance(f,str): # open file if f is str
			f = open(f,'w')
			self.toFile(f) # write self to opened file ...
			f.close()
			return # ... and return
		if self.isWritten: # if already written, do nothing
			return
		for dep in deps: # write dependencies
			dep.toFile(f)
		self.isWritten = True
		f.write(str(self)+'\n') # write self



class Vertex(T3DObject):
	"""Class representing T3D Vertex
	
	:param Vector3 xyz: position
	:param Vector3 fixed: fixed parameter
	:param Vector3 weight: weight
	:param \*\*kw: keywords for :class:`T3DObject`
	"""
	def __init__(self,
			xyz = None,
			fixed = None,
			weight = None,
			**kw):
		T3DObject.__init__(self,**kw)
		self.xyz = xyz
		"""position"""
		self.fixed = fixed
		"""fixed parameter"""
		self.weight = weight
		"""weight"""
		if self.id == -1:
			global NVERTICES
			NVERTICES += 1
			self.id = NVERTICES
		self.curves = []
		"""list of curves sharing the :class:`Vertex`"""
	def __str__(self):
		"""Returns string representation of receiver"""
		ret = 'vertex %d'%self.id
		if self.xyz is not None:
			ret += ' xyz %.8e %.8e %.8e'%tuple(self.xyz[i] for i in (0,1,2))
		if self.fixed is not None:
			for f in self.fixed:
				if isinstance(f,Vertex):
					ret += ' fixed vertex %d'%f.id
				elif isinstance(f,(tuple,list)):
					if isinstance(f[0],Curve):
						ret += ' fixed curve %d t %.8e'%(f[0].id,f[1])
					elif isinstance(f[0],Surface):
						ret += ' fixed surface %d uv %.8e %.8e'%(f[0].id,f[1],f[2])
					else:
						raise PyT3DError, "Illegal value of 'fixed' attribute"
				else:
					raise PyT3DError, "Illegal value of 'fixed' attribute"
		if self.weight is not None:
			ret += ' weight %.8e'%self.weight
		ret += T3DObject.__str__(self)
		return ret
	def toFile(self,f):
		"""Write receiver to given file

		:param file|str f: opened file for writing or file name
		"""
		T3DObject.toFile(self,f)
	def isNeighborOf(self,anotherVertex):
		"""Checks if receiver and given vertex are neighbors (if they are connected with :class:`Curve`)

		:param anotherVertex: given :class:`Vertex`
		:type anotherVertex: :class:`Vertex`
		:rtype: bool
		"""
		for c in self.curves:
			if anotherVertex in c.giveVertices(): return True
		return False
	def giveNeighbors(self):
		"""Returns list of neighbors (:class:`Vectices <Vertex>` which are connected with receiver by :class:`Curves`)

		:rtype: [:class:`Vertex`]
		"""
		return [c.giveSecondVertex(self) for c in self.curves]
	def findCurveTo(self,anotherVertex):
		"""Returns :class:`Curve` connecting receiver and given vertex. Returns None if no such :class:`Curve` exists

		:param anotherVertex: given vertex
		:type anotherVertex: :class:`Vertex`
		:rtype: :class:`Curve`
		"""
		for c in self.curves:
			if anotherVertex in c.giveVertices(): return c
		return None

class Polygon(object):
	"""Class representing T3D :class:`Curve` control point

	:param Vector3 xyz: position
	:param poly: already specified :class:`Polygon`
	:type poly: :class:`Polygon`
	:param size: size
	:param float weight: control point weight
	"""
	def __init__(self,xyz=None,poly=None,size=None,weight=1.):
		self.xyz = xyz
		"""Position"""
		self.poly = poly
		"""Already defined polygon"""
		self.size = size
		"""size parameter"""
		self.weight = weight
		"""weight parameter"""
		self.id = -1
		"""id"""
	def __str__(self):
		"""Returns string representation of receiver"""
		ret = 'polygon %d'%self.id
		if self.xyz is not None:
			ret += ' xyz %.8e %.8e %.8e'%tuple(self.xyz[i] for i in (0,1,2))
		elif self.poly:
			ret += 	' poly %d'%self.poly.id
		else:
			raise PyT3DError, "Either 'xyz' or 'poly' attribute of Polygon has to be specified"
		if self.size is not None:
			ret += ' %s'%size2str(self.size)
		ret += ' weight %.8e'%self.weight
		return ret

class Curve(T3DObject):
	"""class representing T3D Curve object connecting two :class:`Vertices <Vertex>`
	
	:param v1: 1st Vertex
	:type v1: :class:`Vertex`
	:param v2: 2nd Vertex
	:type v2: :class:`Vertex`
	:param polygons: list of control points
	:type polygons: [:class:`Polygon`]
	:param \*\*kw: keywords for :class:`T3DObject`
	""" # TODO
	def __init__(self,v1,v2,
			polygons = [],
			#density
			#rate
			#nosplit
			#polysize
			#count
			#bassoc
			#swap
			#smear
			#mirror
			#duplicate
			#equidistant
			#transition
			**kw):
		T3DObject.__init__(self,**kw)
		self.v1 = v1
		"""1st :class:`Vertex`"""
		self.v2 = v2
		"""2nd :class:`Vertex`"""
		for v in (v1,v2):
			v.curves.append(self)
		self.v2.curves.append(self)
		self.polygons = [p for p in polygons]
		"""list of :class:`Polygons <Polygon>`"""
		if self.id == -1:
			global NCURVES
			NCURVES += 1
			self.id = NCURVES
		self.surfaces = []
		"""list od :class:`Surfaces <Surface>`"""
		self.patches = []
		"""list od :class:`Patches <Patch>`"""
	def __str__(self):
		"""Returns string representation of receiver"""
		ret = 'curve %d vertex %d %d'%(self.id,self.v1.id,self.v2.id)
		if self.polygons:
			ret += ' order %d'%(len(self.polygons)+2)
		ret += T3DObject.__str__(self)
		for i,p in enumerate(self.polygons):
			p.id = i+1
			ret += '\n'+str(p)
		return ret
	def toFile(self,f):
		"""Write receiver to file (its :class:`Vertices <Vertex>` first)

		:param file|str f: opened file for writing or file name
		"""
		T3DObject.toFile(self,f,[self.v1,self.v2])
	def giveVertices(self):
		"""Returns list of receiver's :class:`Vertices <Vertex>`

		:rtype: [:class:`Vertex`]
		"""
		return (self.v1,self.v2)
	def giveSecondVertex(self,v):
		"""Returns secong :class:`Vertex` of receiver (None if given :class:`Vertex` does not belong to receiver)

		:param v: given :class:`Vertex`
		:type v: :class:`Vertex`
		:rtype: :class:`Vertex`
		"""
		return self.v1 if v is self.v2 else self.v2 if v is self.v1 else self.v1 if v is self.v1 and v is self.v2 else None
	def hasCommonVertexWith(self,anotherCurve):
		"""Returns if receiver and given :class:`Curve` has common :class:`Vertex` or not

		:param anotherCurve: given :class:`Curve`
		:type anotherCurve: :class:`Curve`
		:rtype: bool
		"""
		return True if self.v1 in anotherCurve.giveVertices() or self.v2 in anotherCurve.giveVertices() else False
	def findCommonVertexWith(self,anotherCurve):
		"""Returns common vectex with given Curve (None if there is no common Vertex)

		:param anotherCurve: given :class:`Curve`
		:type anotherCurve: :class:`Curve`
		:rtype: :class:`Vertex`
		"""
		if self.v1 in anotherCurve.giveVertices():
			return self.v1
		if self.v2 in anotherCurve.giveVertices():
			return self.v2
		return None


class Entity2d(T3DObject):
	"""Base class for 2D objects (:class:`Patches <Patch>` and :class:`Surfaces <Surface>`)
	
	:param curves: bounding curves
	:type curves: [:class:`Curve`]
	:type bool reorder: do automatic reorder of given curves (default) or not
	:param \*\*kw: keywords for :class:`T3DObject`
	"""
	def __init__(self,curves,reorder=True,**kw):
		self.curves = [c for c in curves]
		"""list of :class:`Curves <Curve>`"""
		if isinstance(self,Surface) and len(self.curves)!=4:
			raise PyT3DError, "Surface has to have exactly 4 bounding curves (%d given)"%len(self.curves)
		T3DObject.__init__(self,**kw)
		self.checkCurves(reorder)
	def giveNeighbors(self):
		"""Returns list of neighbors (2d objects with common boundary :class:`Curves <Curve>`)

		:rtype: [:class:`Entity2d`]
		"""
		return sum(([s for s in c.surfaces if s is not self] + [p for p in c.patches if p is not self] for c in self.curves),[])
	def findCommonCurveWith(self,e):
		"""Returns common :class:`Curve` with given 2d object (None if there is non such :class:`Curve`)

		:param e: given 2d object
		:type e: :class:`Entity2d`
		:rtype: :class:`Curve`
		"""
		for c in self.curves:
			if c in e.curves: return c
		return None
	def giveSignOf(self,c):
		"""Returns sign of given bounding :class:`Curve`

		:param b: given bounding :class:`Curve`
		:type b: :class:`Curve`
		"""
		ret = PLUS if self.curvesSigns[self.curves.index(c)]==PLUS else MINUS if self.curvesSigns[self.curves.index(c)]==MINUS else None
		if isinstance(self,Surface):
			ret = PLUS if ret==MINUS else MINUS if ret==PLUS else None
		return ret
	def checkCurves(self,reorder=True):
		"""Checks if order of curves is correct and reorder them (if reorder=True) accordingly

		:param bool reorder: passing this argument as False means not to reorder the curves, only checks if it is possible to connect them and raise error if not
		"""
		self.curvesSigns = [PLUS] # signs of bounding curves, start with the first curve
		newCurves = [self.curves[0]] # new order of bounding curves
		self.pts = [self.curves[0].v2] # "bounding points", starting with the second vertex of the first curve
		endVertex = self.curves[0].v2 # 
		for i in xrange(len(self.curves)-1): # for all curves ...
			pot1 = [c for c in endVertex.curves if c in self.curves and c not in newCurves] # choose potential next curves according to endVertex
			pot2 = [c for c in pot1 if c.v1 is c.v2] # if there is a curve with identical vertices, use it with priority
			c = pot2[0] if pot2 else pot1[0] # choose actual next curve
			endVertex = c.giveSecondVertex(endVertex) # shift endVertex
			self.curvesSigns.append(MINUS if endVertex is c.v1 else PLUS) # determine sign of curve
			newCurves.append(c) # append new curve
			self.pts.append(endVertex) # append endVertex
		if endVertex is not self.curves[0].v1: # at the end, endVertex has to be the first vertex of the first curve
			raise PyT3DError, "Given curves could not be closed"
		if reorder: # actual reorder
			self.curves = newCurves
	def toFile(self,f):
		"""Write receiver to file (its :class:`Curves <Curve>` first)

		:param file|str f: opened file for writing or file name
		"""
		T3DObject.toFile(self,f,self.curves)


class Patch(Entity2d):
	"""class representing T3D Patch object

	:param curves: bounding curves
	:type curves: [:class:`Curve`]
	:param Vector3 normal: patch normal
	:param Vector3 ref: patch ref parameter (only used if parameter normal is not specified)
	:param \*\*kw: keywords for :class:`Entity2d`
	"""
	def __init__(self,curves,normal=None,ref=None,**kw):
		Entity2d.__init__(self,curves,**kw)
		for c in self.curves:
			c.patches.append(self)
		self.normal = normal
		"""normal vector"""
		self.ref = ref
		"""ref parameter"""
		if normal is None and ref is None:
			c = self.curves
			v1 = c[0].v1 if self.curvesSigns[0]==PLUS else c[0].v2
			v2 = c[0].v2 if v1 is c[0].v1 else c[0].v1
			v3 = c[-1].v2 if v1 is c[-1].v1 else c[-1].v1
			v1 = Vector3(v1.xyz)
			v2 = Vector3(v2.xyz)
			v3 = Vector3(v3.xyz)
			self.normal = ((v2-v1).cross(v3-v1)).normalized()
		if self.id == -1:
			global NPATCHES
			NPATCHES += 1
			self.id = NPATCHES
	def __str__(self):
		"""Returns string representation of receiver"""
		ret = 'patch %d boundary curve '%self.id + ' '.join('%s%d'%(s if s==MINUS else '',c.id) for s,c in zip(self.curvesSigns,self.curves))
		if self.normal is not None:
			ret += ' normal %.8e %.8e %.8e'%tuple(self.normal[i] for i in (0,1,2))
		elif self.ref is not None:
			ret += ' ref %.8e %.8e %.8e'%tuple(self.ref[i] for i in (0,1,2))
		else:
			raise PyT3DError, "Either 'normal' or 'ref' parameter has to be defined"
		ret += T3DObject.__str__(self)
		return ret


class Polygon2(object):
	"""Class representing T3D :class:`Surface` control point

	:param float u: u parameter
	:param float v: v parameter
	:param Vector3 xyz: position
	:param poly: already specified :class:`Polygon2`
	:type poly: :class:`Polygon2`
	:param size: size
	:param float weight: control point weight
	"""
	def __init__(self,u,v,xyz=None,poly=None,size=None,weight=1.):
		self.u = u
		self.v = v
		self.xyz = xyz
		self.poly = poly
		self.size = size
		self.weight = weight
	def __str__(self):
		"""Returns string representation of receiver"""
		ret = 'polygon %d %d'%(self.u,self.v)
		if self.xyz is not None:
			ret += ' xyz %.8e %.8e %.8e'%tuple(self.xyz[i] for i in (0,1,2))
		elif self.poly:
			ret += 	' poly %d %d'%(self.poly.u,self.poly.v)
		else:
			raise PyT3DError, "Either 'xyz' or 'poly' attribute of Polygon2 has to be specified"
		if self.size is not None:
			ret += ' %s'%size2str(self.size)
		ret += ' weight %.8e'%self.weight
		return ret

class Surface(Entity2d):
	"""class representing T3D Surface object

	:param curves: bounding curves
	:type curves: [:class:`Curve`]
	:param (int,int) order: order in u and v direction
	:param polygons: list of control points
	:type polygons: [:class:`Polygon`]
	:param \*\*kw: keywords for :class:`Entity2d`
	"""
	def __init__(self,curves,order=(1,1),polygons=[],
			**kw):
		Entity2d.__init__(self,curves,**kw)
		self.order = order
		"""order in u and v direction"""
		self.polygons = [p for p in polygons]
		"""list of :class:`Polygons <Polygon2>`"""
		for c in self.curves:
			c.surfaces.append(self)
		if self.id == -1:
			global NSURFACES
			NSURFACES += 1
			self.id = NSURFACES
	def __str__(self):
		"""Returns string representation of receiver"""
		ret = 'surface %d curve %d %d %d %d'%tuple(o.id for o in (self,)+tuple(self.curves))
		if self.order != (1,1):
			ret += ' order %d %d'%self.order
		ret += T3DObject.__str__(self)
		for p in self.polygons:
			ret += '\n'+str(p)
		return ret


class Region(T3DObject):
	"""Class representing T3D Region

	:param surfaces: bounding :class:`Surfaces <Surface>`
	:type surface: [:class:`Surface`]
	:param patches: bounding :class:`Patches <Patch>`
	:type surface: [:class:`Patch`]
	:param \*\*kw: keywords for :class:`T3DObject`
	"""
	def __init__(self,surfaces=[],patches=[],**kw):
		T3DObject.__init__(self,**kw)
		self.surfaces = [s for s in surfaces]
		"""list of :class:`Surfaces <Surface>`"""
		self.patches = [p for p in patches]
		"""list of :class:`Patches <Patch>`"""
		self.checkBoundaries()
		self.subregions = []
		"""list of :class:`Regions <Region>`"""
		if self.id == -1:
			global NREGIONS
			NREGIONS += 1
			self.id = NREGIONS
	def __str__(self):
		"""Returns string representation of receiver"""
		ret = 'region %d'%(self.id)
		if self.surfaces:
			ret += ' boundary surface'
			for i in range(len(self.surfaces)):
				ret += ' %s'%('-'+str(self.surfaces[i].id) if self.surfacesSigns[i]==MINUS else str(self.surfaces[i].id))
		if self.patches:
			ret += ' boundary patch'
			for i in range(len(self.patches)):
				ret += ' %s'%('-'+str(self.patches[i].id) if self.patchesSigns[i]==MINUS else str(self.patches[i].id))
		ret += T3DObject.__str__(self)
		for s in self.subregions:
			ret += ' subregion %d'%s.id
		return ret+'\n'
	def giveSignOf(self,b):
		"""Returns sign of given bounding :class:`Entity2d`

		:param b: given bounding :class:`Entity2d`
		:type b: :class:`Entity2d`
		"""
		if isinstance(b,Surface):
			return PLUS if self.surfacesSigns[self.surfaces.index(b)]==PLUS else MINUS if self.surfacesSigns[self.surfaces.index(b)]==MINUS else None
		elif isinstance(b,Patch):
			return PLUS if self.patchesSigns[self.patches.index(b)]==PLUS else MINUS if self.patchesSigns[self.patches.index(b)]==MINUS else None
	def _makeMinusOrientation(self,b):
		"""Set minus orientation of given bounding :class:`Entity2d`

		:param b: given bounding :class:`Entity2d`
		:type b: :class:`Entity2d`
		"""
		if isinstance(b,Surface):
			self.surfacesSigns[self.surfaces.index(b)] = MINUS
		elif isinstance(b,Patch):
			self.patchesSigns[self.patches.index(b)]   = MINUS
	def checkOrientationOf(self,b1,b2):
		"""Checks orientation of two receiver's bounding :class:`Entities2d <Entity2d>`. Changes sign of b2 or raise error in case of problem. Returns if the second boundary was changed or not

		:param b1: 1st bounding :class:`Entity2d`
		:type b1: :class:`Entity2d`
		:param b2: 2nd bounding :class:`Entity2d`
		:type b2: :class:`Entity2d`
		:rtype: bool
		"""
		c = b1.findCommonCurveWith(b2) # common curve
		# signs of common curve from point of entities2d
		ac1 = b1.giveSignOf(c) 
		ac2 = b2.giveSignOf(c)
		# reverse the signs of c if bounding entities2d has minus sign
		if self.giveSignOf(b1)==MINUS:
			ac1 = PLUS if ac1==MINUS else MINUS if ac1==PLUS else None
		if self.giveSignOf(b2)==MINUS:
			ac2 = PLUS if ac2==MINUS else MINUS if ac2==PLUS else None
		# if ((ac1==PLUS and ac2==MINUS) or (ac1==MINUS and ac2==PLUS)): everything is ok
		if not ((ac1==PLUS and ac2==MINUS) or (ac1==MINUS and ac2==PLUS)): # else sign of b2 should be be reversed ...
			if self.giveSignOf(b2)==MINUS: # ... if it is minus already, it was reversed according to some previous boundary and there is some problem with the boundaries
				raise PyT3DError, "Bounding entities2d could not be closed"
			self._makeMinusOrientation(b2) # ... else make it minus form plus
			return True
		return False
	def checkBoundaries(self,nSteps=10):
		"""Checks and reorder receiver's 2d entities
		
		:param int nSteps: maximum number of tries + number of boundaries
		"""
		# make sign of all boundaries plus
		self.surfacesSigns = [PLUS for s in self.surfaces]
		self.patchesSigns = [PLUS for p in self.patches]
		# all boundaries
		bs = self.surfaces + self.patches
		nSteps += len(bs)
		#
		someSkip = True
		skipCounter = 0
		while someSkip:
			someSkip = False # start from beginning
			for i,b1 in enumerate(bs): # for all boundary surfaces
				if someSkip: # breaks the outer loop (someSkip is set True in the inner loop)
					break
				for b2 in b1.giveNeighbors(): # for all its neighbors
					if not (b2 in self.surfaces or b2 in self.patches): # b2 not in region's boundaries
						continue
					if bs.index(b2) < i: # the pair aready checked
						continue
					# check b1 and b2
					if self.checkOrientationOf(b1,b2): # if b2 is changed, break both loops over boundaries and check from the beginning
						someSkip = True
						break
			skipCounter += 1
			if skipCounter > nSteps:
				raise PyT3DError, "Boundaries of region could not be ordered in %d tries"%nSteps
	def reverse(self):
		"""Reverse receiver (sign of its bounding :class:`Entities2d <Entity2d>`"""
		for i in range(len(self.surfacesSigns)):
			self.surfacesSigns[i]=MINUS if self.surfacesSigns[i]==PLUS else PLUS
		for i in range(len(self.patchesSigns)):
			self.patchesSigns[i]=MINUS if self.patchesSigns[i]==PLUS else PLUS
	def toFile(self,f):
		"""Write receiver to file (its surfaces, patches and subregions first)

		:param file|str f: opened file for writing or file name
		"""
		T3DObject.toFile(self,f,self.surfaces+self.patches+self.subregions)
		



def run(iName,oName='',graphics=False,exe='t3d',defaultSize=1.,extra='',phaseByPhase=False):
	"""Runs T3D

	:param str iName: input file name
	:param str oName: output file name
	:param bool graphics: if run T3D with graphics or not
	:param str exe: T3D executable name
	:param float defaultSize: default size
	:param str extra: extra command lines arguments for T3D
	:param bool phaseByPhase: run T3D phase by phase or not
	"""
	import subprocess
	cmd = [exe,'-i',iName,'-d',str(defaultSize)]
	if oName:
		cmd.extend(('-o',oName))
	if graphics:
		cmd.append('-X')
		if phaseByPhase:
			cmd.append('-@')
	if extra:
		if isinstance(extra,str):
			cmd.extend(extra.split())
		elif isinstance(extra,(list,tuple)):
			cmd.extend(extra)
		else:
			raise RuntimeError, "Extra parameter has to be str, tuple or list"
	subprocess.Popen(cmd).wait()

def axisAlignedBox(lower,upper):
	"""Construct axis aligned box

	:param Vector3 lower: lower corner
	:param Vector3 upper: upper corner
	:rtype: ([Vertex],[Curve],[Surface],Region)
	"""
	lower,upper = Vector3(lower),Vector3(upper)
	d = upper - lower
	vertices = [Vertex(xyz=lower+Vector3(x*d[0],y*d[1],z*d[2])) for x,y,z in ((0,0,0),(1,0,0),(0,1,0),(1,1,0),(0,0,1),(1,0,1),(0,1,1),(1,1,1))]
	curves = [Curve(vertices[i],vertices[j]) for i,j in ((0,1),(1,3),(3,2),(2,0),(0,4),(1,5),(3,7),(2,6),(4,5),(5,7),(7,6),(6,4))]
	surfaces = [Surface((curves[i],curves[j],curves[k],curves[l])) for i,j,k,l in ((0,1,2,3),(0,5,8,4),(1,6,9,5),(2,7,10,6),(3,4,11,7),(8,9,10,11))]
	region = Region(surfaces=surfaces)
	return vertices,curves,surfaces,region

def box(vertices):
	"""Returns box

	:param [Vertex] vertices: given vertices
	:rtype: ([Vertex],[Curve],[Surface],Region)
	"""
	curves = [Curve(vertices[i],vertices[j]) for i,j in ((0,1),(1,3),(3,2),(2,0),(0,4),(1,5),(3,7),(2,6),(4,5),(5,7),(7,6),(6,4))]
	surfaces = [Surface((curves[i],curves[j],curves[k],curves[l])) for i,j,k,l in ((0,1,2,3),(0,5,8,4),(1,6,9,5),(2,7,10,6),(3,4,11,7),(8,9,10,11))]
	region = Region(surfaces=surfaces)
	return vertices,curves,surfaces,region

def sphere(center,radius): # TODO ori
	"""Returns sphere

	:param Vector3 center: center
	:param float radius: radius
	:rtype: ([Vertex],[Curve],[Surface],Region)
	"""
	c = Vector3(center)
	rx = Vector3(radius,0,0)
	ry = Vector3(0,radius,0)
	rz = Vector3(0,0,radius)
	v1 = Vertex(xyz=c+rx)
	v2 = Vertex(xyz=c-rx)
	w = 1/3.
	c1 = Curve(v1,v2,polygons=[Polygon(xyz=c+rx+2*ry,weight=w),Polygon(xyz=c-rx+2*ry,weight=w)])
	c2 = Curve(v2,v1,polygons=[Polygon(xyz=c-rx-2*ry,weight=w),Polygon(xyz=c+rx-2*ry,weight=w)])
	c3 = Curve(v1,v1,polygons=[Polygon(xyz=c+rx,weight=w),Polygon(xyz=c+rx,weight=w)])
	c4 = Curve(v2,v2,polygons=[Polygon(xyz=c-rx,weight=w),Polygon(xyz=c-rx,weight=w)])
	w = 1/9.
	s1 = Surface((c1,c3,c2,c4),reorder=False,polygons=[
		Polygon2(1,1,c+rx+2*ry+4*rz,weight=w),
		Polygon2(1,2,c+rx-2*ry+4*rz,weight=w),
		Polygon2(2,1,c-rx+2*ry+4*rz,weight=w),
		Polygon2(2,2,c-rx-2*ry+4*rz,weight=w)])
	s2 = Surface((c1,c3,c2,c4),reorder=False,polygons=[
		Polygon2(1,1,c+rx+2*ry-4*rz,weight=w),
		Polygon2(1,2,c+rx-2*ry-4*rz,weight=w),
		Polygon2(2,1,c-rx+2*ry-4*rz,weight=w),
		Polygon2(2,2,c-rx-2*ry-4*rz,weight=w)])
	r = Region(surfaces=(s1,s2))
	return [v1,v2],[c1,c3,c2,c4],[s1,s2],r

def polyhedron(points,connectivity,returnOnlyRegion=False):
	"""Returns polygon

	:param [Vector3] points: polyhedron vertices
	:param [[int]] connectivity: connectivity table of polyhedron
	:param bool returnOnlyRegion: if True, returns only the resulting region
	:rtype: ([Vertex],[Curve],[Patch],Region)|Region
	"""
	vertices = [Vertex(xyz=v) for v in points]
	curves = []
	patches = [None for i in connectivity]
	for i,f in enumerate(connectivity):
		fcurves = [None for j in f]
		for j,i1 in enumerate(f):
			i2 = f[0] if j==len(f)-1 else f[j+1]
			v1,v2 = vertices[i1],vertices[i2]
			if not v1.isNeighborOf(v2):
				curves.append(Curve(v1,v2))
			fcurves[j] = v1.findCurveTo(v2)
		patches[i] = Patch(fcurves)
	region = Region(patches=patches)
	if returnOnlyRegion:
		return region
	return vertices,curves,patches,region
if __name__ == "__main__":
	import test
	test.test()
